// Copyright AudioKit. All Rights Reserved. Revision History at http://github.com/AudioKit/AudioKit/
// This file was auto-autogenerated by scripts and templates at http://github.com/AudioKit/AudioKitDevTools/

import AVFoundation

/// AudioKit version of Apple's HighShelfFilter Audio Unit
///
public class HighShelfFilter: Node {

    fileprivate let effectAU = AVAudioUnitEffect(appleEffect: kAudioUnitSubType_HighShelfFilter)

    let input: Node
    public var connections: [Node] { [input] }
    public var avAudioNode: AVAudioNode { effectAU }

    /// Specification details for cutOffFrequency
    public static let cutOffFrequencyDef = NodeParameterDef(
        identifier: "cutOffFrequency",
        name: "Cut Off Frequency",
        address: 0,
        defaultValue: 10000,
        range: 10000 ... 22050,
        unit: .hertz)

    /// Cut Off Frequency (Hertz) ranges from 10000 to 22050 (Default: 10000)
    @Parameter(cutOffFrequencyDef) public var cutOffFrequency: AUValue

    /// Specification details for gain
    public static let gainDef = NodeParameterDef(
        identifier: "gain",
        name: "Gain",
        address: 1,
        defaultValue: 0,
        range: -40 ... 40,
        unit: .decibels)

    /// Gain (decibels) ranges from -40 to 40 (Default: 0)
    @Parameter(gainDef) public var gain: AUValue

    /// Tells whether the node is processing (ie. started, playing, or active)
    public var isStarted = true

    /// Initialize the high shelf filter node
    ///
    /// - parameter input: Input node to process
    /// - parameter cutOffFrequency: Cut Off Frequency (Hertz) ranges from 10000 to 22050 (Default: 10000)
    /// - parameter gain: Gain (decibels) ranges from -40 to 40 (Default: 0)
    ///
    public init(
        _ input: Node,
        cutOffFrequency: AUValue = cutOffFrequencyDef.defaultValue,
        gain: AUValue = gainDef.defaultValue) {
        self.input = input

        associateParams(with: effectAU)

        self.cutOffFrequency = cutOffFrequency
        self.gain = gain
    }

    /// Function to start, play, or activate the node, all do the same thing
    public func start() {
        effectAU.bypass = false
        isStarted = true
    }

    /// Function to stop or bypass the node, both are equivalent
    public func stop() {
        effectAU.bypass = true
        isStarted = false
    }
}
